//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TfisMailslot.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TfisMailslot *)
{
    new TfisMailslot(NULL);
}
//---------------------------------------------------------------------------
void __fastcall SlotThread::EventWrapper(void)
{
    prNotifyEvent(prNotifyObj, prMessage, prSize, prComputer, prUser);
}
//---------------------------------------------------------------------------
void __fastcall SlotThread::Execute(void)
{
    TfisMailslot *lHostObj = (TfisMailslot *)prNotifyObj;
    while(!Terminated)
    {
        prSize = MAX_MAILSLOT_MSG_SIZE;
        if(lHostObj->ReadMailslot(prMsgBuffer, prSize))
        {
            AnsiString lId;
            prMessage = lHostObj->RemoveHeader(prMsgBuffer, prSize,
                prComputer, prUser, lId);
            if(prMessage != NULL)
            {
                AnsiString lOldId;
                if(!prLastMsgIds->FindString(prUser, lOldId) || lOldId != lId)
                {
                    prLastMsgIds->AddString(prUser, lId);
                    Synchronize(EventWrapper);
                }
            }
        }
    }
}
//---------------------------------------------------------------------------
__fastcall SlotThread::SlotThread(TMailslotMsgEvent aNotifyEvent,
    TObject *aNotifyObj) : TThread(false)
{
    prLastMsgIds = new StringMap();
    prMsgBuffer = new BYTE[MAX_MAILSLOT_MSG_SIZE];
    prNotifyEvent = aNotifyEvent;
    prNotifyObj = aNotifyObj;
}
//---------------------------------------------------------------------------
__fastcall SlotThread::~SlotThread(void)
{
    delete [] prMsgBuffer;
    delete prLastMsgIds;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
__fastcall TfisMailslot::TfisMailslot(TComponent* Owner)
    : TComponent(Owner)
{
    char lName[MAX_COMPUTERNAME_LENGTH + 1];
    DWORD lSize = sizeof(lName);

    if(GetComputerName(lName, &lSize))
    {
        FComputerName = lName;
    }
    lSize = sizeof(lName);
    if(GetUserName(lName, &lSize))
    {
        FUserName = lName;
    }
    FSlotName = "";
    FMailslot = NULL;
    FClientMailslot = NULL;
    FOnMessage = NULL;
    thrSlotThread = NULL;
}
//---------------------------------------------------------------------------
__fastcall TfisMailslot::~TfisMailslot(void)
{
    CloseClientMailslot();
    CloseMailslot();
}
//---------------------------------------------------------------------------
void __fastcall TfisMailslot::SetName(AnsiString aValue)
{
    bool lChange = (ComponentState.Contains(csDesigning) &&
        (Name == FSlotName || FSlotName.Length() == 0));
    TComponent::SetName(aValue);
    if(lChange)
    {
        FSlotName = Name;
    }
}
//---------------------------------------------------------------------------
bool __fastcall TfisMailslot::CreateMailslot(void)
{
    if(FMailslot != NULL) return(false);
    AnsiString lMailslotName = "\\\\.\\mailslot\\" + FSlotName;
    FMailslot = ::CreateMailslot(lMailslotName.c_str(), 0, MAILSLOT_TIMEOUT, NULL);
    if(FMailslot == INVALID_HANDLE_VALUE)
    {
        FMailslot = NULL;
        return(false);
    }
    if(FOnMessage != NULL)
    {
        thrSlotThread = new SlotThread(FOnMessage, this);
    }
    return(true);
}
//---------------------------------------------------------------------------
bool __fastcall TfisMailslot::OpenClientMailslot(AnsiString aTarget,
    AnsiString aName)
{
    AnsiString lMailslotName = "\\\\" + aTarget + "\\mailslot\\" + aName;
    FClientMailslot = CreateFile(lMailslotName.c_str(), GENERIC_WRITE, FILE_SHARE_READ,
        NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(FClientMailslot == INVALID_HANDLE_VALUE)
    {
        FClientMailslot = NULL;
        return(false);
    }
    return(true);
}
//---------------------------------------------------------------------------
bool __fastcall TfisMailslot::CloseMailslot(void)
{
    if(FMailslot != NULL)
    {
        if(thrSlotThread != NULL)
        {
            thrSlotThread->Terminate();
            thrSlotThread->WaitFor();
            delete thrSlotThread;
            thrSlotThread = NULL;
        }
        CloseHandle(FMailslot);
        FMailslot = NULL;
    }
    return(true);
}
//---------------------------------------------------------------------------
bool __fastcall TfisMailslot::CloseClientMailslot(void)
{
    if(FClientMailslot != NULL)
    {
        CloseHandle(FClientMailslot);
        FClientMailslot = NULL;
    }
    return(true);
}
//---------------------------------------------------------------------------
bool __fastcall TfisMailslot::WriteClientMailslot(void *aBuffer, unsigned aNbrBytes)
{
    if(FClientMailslot != NULL)
    {
        DWORD lBytesWritten;
        aBuffer = AddHeader(aBuffer, aNbrBytes);
        bool lResult = WriteFile(FClientMailslot, aBuffer, aNbrBytes, &lBytesWritten, NULL);
        delete [] aBuffer;
        return(lResult);
    }
    return(false);
}
//---------------------------------------------------------------------------
bool __fastcall TfisMailslot::ReadMailslot(void *aBuffer, unsigned &aSize)
{
    if(FMailslot != NULL)
    {
        DWORD lNbrBytesRead;
        if(ReadFile(FMailslot, aBuffer, (DWORD)aSize, &lNbrBytesRead, NULL))
        {
            aSize = lNbrBytesRead;
            return(true);
        }
    }
    return(false);
}
//---------------------------------------------------------------------------
void *__fastcall TfisMailslot::AddHeader(void *aMessage, unsigned &aLength)
{
    AnsiString lId = (double)Now();
    AnsiString lHeader = FComputerName + ";" + FUserName + ";" + lId + ";";
    int lLength = lHeader.Length();
    char *lMessage = new char[aLength + lLength];
    memcpy(lMessage, lHeader.c_str(), lLength);
    memcpy(lMessage + lLength, aMessage, aLength);
    aLength += lHeader.Length();
    return(lMessage);
}
//---------------------------------------------------------------------------
void *__fastcall TfisMailslot::RemoveHeader(void *aMessage, unsigned &aLength,
    AnsiString &aComputerName, AnsiString &aUserName, AnsiString &aId)
{
    AnsiString lBuffer((char *)aMessage, 100);

    int lPos = lBuffer.Pos(";");
    if(lPos == 0) return(NULL);
    aComputerName = lBuffer.SubString(1, lPos - 1);
    lBuffer = lBuffer.SubString(lPos + 1, 255);
    lPos = lBuffer.Pos(";");
    if(lPos == 0) return(NULL);
    aUserName = lBuffer.SubString(1, lPos - 1);
    lBuffer = lBuffer.SubString(lPos + 1, 255);
    lPos = lBuffer.Pos(";");
    if(lPos == 0) return(NULL);
    aId = lBuffer.SubString(1, lPos - 1);
    int lHeaderLength = aComputerName.Length() + aUserName.Length() + aId.Length() + 3;
    aLength -= lHeaderLength;
    return((char *)aMessage + lHeaderLength);
}
//---------------------------------------------------------------------------
namespace Tfismailslot
{
    void __fastcall PACKAGE Register()
    {
        TComponentClass classes[1] = {__classid(TfisMailslot)};
        RegisterComponents("FISH", classes, 0);
    }
}
//---------------------------------------------------------------------------


