//---------------------------------------------------------------------------
#ifndef TfisMailslotH
#define TfisMailslotH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
//---------------------------------------------------------------------------
#define MAX_MAILSLOT_MSG_SIZE       65535
#define MAILSLOT_TIMEOUT            1000
//---------------------------------------------------------------------------
typedef void __fastcall (__closure *TMailslotMsgEvent)(System::TObject* Sender,
    void *Message, unsigned Length, AnsiString ComputerName, AnsiString UserName);
//---------------------------------------------------------------------------
class StringMap : public TStringList
{
public:
    void __fastcall DeleteString(const AnsiString aIndex)
    {
        int lIndex = TStringList::IndexOf(aIndex);
        if(lIndex != -1)
        {
            delete (AnsiString *)(Objects[lIndex]);
            TStringList::Delete(lIndex);
        }
    }
    int __fastcall AddString(const AnsiString aIndex, const AnsiString aCargo)
    {
        DeleteString(aIndex);
        return(AddObject(aIndex, (TObject *)new AnsiString(aCargo)));
    }
    bool __fastcall FindString(const AnsiString aIndex, AnsiString &aCargo)
    {
        int lIndex = TStringList::IndexOf(aIndex);
        if(lIndex != -1)
        {
            aCargo = *(AnsiString *)(Objects[lIndex]);
            return(true);
        }
        return(false);
    }
    __fastcall ~StringMap()
    {
        for(int lCount = 0; lCount < Count; lCount++)
        {
            delete (AnsiString *)(Objects[lCount]);
        }
    }
};
//---------------------------------------------------------------------------
class SlotThread : public TThread
{
private:
    StringMap *prLastMsgIds;
    BYTE *prMsgBuffer;
    unsigned prSize;
    void *prMessage;
    AnsiString prComputer, prUser;
    TObject *prNotifyObj;
    TMailslotMsgEvent prNotifyEvent;
    void __fastcall EventWrapper(void);
protected:
    void __fastcall Execute(void);
public:
    __fastcall SlotThread(TMailslotMsgEvent aNotifyEvent, TObject *aNotifyObj);
    __fastcall ~SlotThread(void);
};
//---------------------------------------------------------------------------
class PACKAGE TfisMailslot : public TComponent
{
    friend void __fastcall SlotThread::Execute(void);
private:
    AnsiString FSlotName, FComputerName, FUserName;
    HANDLE FMailslot, FClientMailslot;
    TMailslotMsgEvent FOnMessage;
    SlotThread *thrSlotThread;
    bool __fastcall ReadMailslot(void *aBuffer, unsigned &aSize);
    void *__fastcall AddHeader(void *aMessage, unsigned &aLength);
    void *__fastcall RemoveHeader(void *aMessage, unsigned &aLength,
        AnsiString &aComputerName, AnsiString &aUserName, AnsiString &aId);
protected:
    virtual void __fastcall SetName(AnsiString aValue);
public:
    bool __fastcall CreateMailslot(void);
    bool __fastcall OpenClientMailslot(AnsiString aTarget, AnsiString aName);
    bool __fastcall CloseMailslot(void);
    bool __fastcall CloseClientMailslot(void);
    bool __fastcall WriteClientMailslot(void *aBuffer, unsigned aNbrBytes);
    __fastcall TfisMailslot(TComponent* Owner);
    __fastcall ~TfisMailslot(void);
    __property HANDLE Mailslot = {read = FMailslot};
    __property HANDLE ClientMailslot = {read = FClientMailslot};
    __property AnsiString ComputerName = {read = FComputerName};
    __property AnsiString UserName = {read = FUserName};
__published:
    __property AnsiString SlotName = {read = FSlotName, write = FSlotName};
    __property TMailslotMsgEvent OnMessage = {read = FOnMessage, write = FOnMessage};
};
//---------------------------------------------------------------------------
#endif
